(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.VirtualList = factory());
}(this, (function () { 'use strict';

	function noop() {}

	function assign(tar, src) {
		for (var k in src) tar[k] = src[k];
		return tar;
	}

	function appendNode(node, target) {
		target.appendChild(node);
	}

	function insertNode(node, target, anchor) {
		target.insertBefore(node, anchor);
	}

	function detachNode(node) {
		node.parentNode.removeChild(node);
	}

	function createElement(name) {
		return document.createElement(name);
	}

	function addListener(node, event, handler) {
		node.addEventListener(event, handler, false);
	}

	function removeListener(node, event, handler) {
		node.removeEventListener(event, handler, false);
	}

	function setAttribute(node, attribute, value) {
		node.setAttribute(attribute, value);
	}

	function setStyle(node, key, value) {
		node.style.setProperty(key, value);
	}

	function destroyBlock(block, lookup) {
		block.u();
		block.d();
		lookup[block.key] = null;
	}

	function outroAndDestroyBlock(block, lookup) {
		block.o(function() {
			destroyBlock(block, lookup);
		});
	}

	function updateKeyedEach(old_blocks, component, changed, key_prop, dynamic, list, lookup, node, has_outro, create_each_block, intro_method, get_context) {
		var o = old_blocks.length;
		var n = list.length;

		var i = o;
		var old_indexes = {};
		while (i--) old_indexes[old_blocks[i].key] = i;

		var new_blocks = [];
		var new_lookup = {};
		var deltas = {};

		var i = n;
		while (i--) {
			var key = list[i][key_prop];
			var block = lookup[key];

			if (!block) {
				block = create_each_block(component, key, get_context(i));
				block.c();
			} else if (dynamic) {
				block.p(changed, get_context(i));
			}

			new_blocks[i] = new_lookup[key] = block;

			if (key in old_indexes) deltas[key] = Math.abs(i - old_indexes[key]);
		}

		var next = null;

		var will_move = {};
		var did_move = {};

		var destroy = has_outro ? outroAndDestroyBlock : destroyBlock;

		function insert(block) {
			block[intro_method](node, next && next.first);
			next = lookup[block.key] = block;
			n--;
		}

		while (o && n) {
			var new_block = new_blocks[n - 1];
			var old_block = old_blocks[o - 1];
			var new_key = new_block.key;
			var old_key = old_block.key;

			if (new_block === old_block) {
				// do nothing
				next = new_block;
				o--;
				n--;
			}

			else if (!new_lookup[old_key]) {
				// remove old block
				destroy(old_block, lookup);
				o--;
			}

			else if (!lookup[new_key] || will_move[new_key]) {
				insert(new_block);
			}

			else if (did_move[old_key]) {
				o--;

			} else if (deltas[new_key] > deltas[old_key]) {
				did_move[new_key] = true;
				insert(new_block);

			} else {
				will_move[old_key] = true;
				o--;
			}
		}

		while (o--) {
			var old_block = old_blocks[o];
			if (!new_lookup[old_block.key]) destroy(old_block, lookup);
		}

		while (n) insert(new_blocks[n - 1]);

		return new_blocks;
	}

	function blankObject() {
		return Object.create(null);
	}

	function destroy(detach) {
		this.destroy = noop;
		this.fire('destroy');
		this.set = this.get = noop;

		if (detach !== false) this._fragment.u();
		this._fragment.d();
		this._fragment = this._state = null;
	}

	function _differs(a, b) {
		return a != a ? b == b : a !== b || ((a && typeof a === 'object') || typeof a === 'function');
	}

	function dispatchObservers(component, group, changed, newState, oldState) {
		for (var key in group) {
			if (!changed[key]) continue;

			var newValue = newState[key];
			var oldValue = oldState[key];

			var callbacks = group[key];
			if (!callbacks) continue;

			for (var i = 0; i < callbacks.length; i += 1) {
				var callback = callbacks[i];
				if (callback.__calling) continue;

				callback.__calling = true;
				callback.call(component, newValue, oldValue);
				callback.__calling = false;
			}
		}
	}

	function fire(eventName, data) {
		var handlers =
			eventName in this._handlers && this._handlers[eventName].slice();
		if (!handlers) return;

		for (var i = 0; i < handlers.length; i += 1) {
			handlers[i].call(this, data);
		}
	}

	function get(key) {
		return key ? this._state[key] : this._state;
	}

	function init(component, options) {
		component._observers = { pre: blankObject(), post: blankObject() };
		component._handlers = blankObject();
		component._bind = options._bind;

		component.options = options;
		component.root = options.root || component;
		component.store = component.root.store || options.store;
	}

	function observe(key, callback, options) {
		var group = options && options.defer
			? this._observers.post
			: this._observers.pre;

		(group[key] || (group[key] = [])).push(callback);

		if (!options || options.init !== false) {
			callback.__calling = true;
			callback.call(this, this._state[key]);
			callback.__calling = false;
		}

		return {
			cancel: function() {
				var index = group[key].indexOf(callback);
				if (~index) group[key].splice(index, 1);
			}
		};
	}

	function on(eventName, handler) {
		if (eventName === 'teardown') return this.on('destroy', handler);

		var handlers = this._handlers[eventName] || (this._handlers[eventName] = []);
		handlers.push(handler);

		return {
			cancel: function() {
				var index = handlers.indexOf(handler);
				if (~index) handlers.splice(index, 1);
			}
		};
	}

	function set(newState) {
		this._set(assign({}, newState));
		if (this.root._lock) return;
		this.root._lock = true;
		callAll(this.root._beforecreate);
		callAll(this.root._oncreate);
		callAll(this.root._aftercreate);
		this.root._lock = false;
	}

	function _set(newState) {
		var oldState = this._state,
			changed = {},
			dirty = false;

		for (var key in newState) {
			if (this._differs(newState[key], oldState[key])) changed[key] = dirty = true;
		}
		if (!dirty) return;

		this._state = assign(assign({}, oldState), newState);
		this._recompute(changed, this._state);
		if (this._bind) this._bind(changed, this._state);

		if (this._fragment) {
			dispatchObservers(this, this._observers.pre, changed, this._state, oldState);
			this._fragment.p(changed, this._state);
			dispatchObservers(this, this._observers.post, changed, this._state, oldState);
		}
	}

	function callAll(fns) {
		while (fns && fns.length) fns.shift()();
	}

	function _mount(target, anchor) {
		this._fragment[this._fragment.i ? 'i' : 'm'](target, anchor || null);
	}

	function _unmount() {
		if (this._fragment) this._fragment.u();
	}

	var proto = {
		destroy: destroy,
		get: get,
		fire: fire,
		observe: observe,
		on: on,
		set: set,
		teardown: destroy,
		_recompute: noop,
		_set: _set,
		_mount: _mount,
		_unmount: _unmount,
		_differs: _differs
	};

	/* src/index.html generated by Svelte v1.60.0 */

	function visible(items, start, end) {
		return items.slice(start, end).map((data, i) => {
			return { index: i + start, data };
		});
	}

	function data() {
		return { top: 0, bottom: 0, start: 0, end: 0 };
	}
	var methods = {
		handleScroll() {
			const { items, start, end } = this.get();
			const { offsetHeight, scrollTop } = this.refs.viewport;

			let paddingTop = 0;
			let offset = 0;
			let i = 0;

			for (let v = 0; v < this.rows.length; v += 1) {
				this.heightMap[start + v] = this.rows[v].offsetHeight;
			}

			for (; i < items.length; i += 1) {
				offset += this.heightMap[i];
				if (offset > scrollTop) break;

				paddingTop = offset;
			}

			const newStart = i++;

			for (; i < items.length; i += 1) {
				if (offset > scrollTop + offsetHeight) break;
				offset += this.heightMap[i];
			}

			const newEnd = i;

			if (newStart === start && newEnd === end) return;

			let paddingBottom = 0;
			for (; i < items.length; i += 1) paddingBottom += this.heightMap[i];

			this.set({
				top: paddingTop,
				bottom: paddingBottom,
				start: newStart,
				end: newEnd
			});
		}
	};

	function oncreate() {
		const { items, visible } = this.get();
		const { viewport, container } = this.refs;
		const viewportHeight = viewport.offsetHeight;

		this.rows = container.getElementsByClassName('row');
		this.heightMap = [];

		let height = 0;
		let i = 0;

		while (height < viewportHeight && i < items.length) {
			this.set({ end: i + 1 });

			const rowHeight = this.heightMap[i] = this.rows[i].offsetHeight;
			height += rowHeight;

			i += 1;
		}

		const end = i;
		const avg = Math.round(height / i);

		for (; i < items.length; i += 1) this.heightMap[i] = avg;

		this.set({
			bottom: (items.length - end) * avg
		});
	}
	function add_css() {
		var style = createElement("style");
		style.id = 'svelte-1hy6rmt-style';
		style.textContent = "[svelte-ref-viewport].svelte-1hy6rmt{height:100%;overflow-y:auto;-webkit-overflow-scrolling:touch}.row.svelte-1hy6rmt{overflow:hidden}";
		appendNode(style, document.head);
	}

	function create_main_fragment(component, state) {
		var div, div_1, each_blocks_1 = [], each_lookup = blankObject();

		var each_value = state.visible;

		for (var i = 0; i < each_value.length; i += 1) {
			var key = each_value[i].index;
			each_blocks_1[i] = each_lookup[key] = create_each_block(component, key, assign(assign({}, state), {
				each_value: each_value,
				item: each_value[i],
				item_index: i
			}));
		}

		function scroll_handler(event) {
			component.handleScroll();
		}

		return {
			c: function create() {
				div = createElement("div");
				div_1 = createElement("div");

				for (i = 0; i < each_blocks_1.length; i += 1) each_blocks_1[i].c();
				this.h();
			},

			h: function hydrate() {
				setAttribute(div, "svelte-ref-viewport", "");
				setStyle(div_1, "padding-top", "" + state.top + "px");
				setStyle(div_1, "padding-bottom", "" + state.bottom + "px");
				addListener(div, "scroll", scroll_handler);
				div.className = "svelte-1hy6rmt";
			},

			m: function mount(target, anchor) {
				insertNode(div, target, anchor);
				appendNode(div_1, div);

				for (i = 0; i < each_blocks_1.length; i += 1) each_blocks_1[i].m(div_1, null);

				component.refs.container = div_1;
				component.refs.viewport = div;
			},

			p: function update(changed, state) {
				var each_value = state.visible;

				each_blocks_1 = updateKeyedEach(each_blocks_1, component, changed, "index", 1, each_value, each_lookup, div_1, false, create_each_block, "m", function(i) {
					return assign(assign({}, state), {
						each_value: each_value,
						item: each_value[i],
						item_index: i
					});
				});

				if (changed.top) {
					setStyle(div_1, "padding-top", "" + state.top + "px");
				}

				if (changed.bottom) {
					setStyle(div_1, "padding-bottom", "" + state.bottom + "px");
				}
			},

			u: function unmount() {
				detachNode(div);
			},

			d: function destroy$$1() {
				for (i = 0; i < each_blocks_1.length; i += 1) each_blocks_1[i].d();

				if (component.refs.container === div_1) component.refs.container = null;
				removeListener(div, "scroll", scroll_handler);
				if (component.refs.viewport === div) component.refs.viewport = null;
			}
		};
	}

	// (3:2) {{#each visible as item @index}}
	function create_each_block(component, key_1, state) {
		var item = state.item, each_value = state.each_value, item_index = state.item_index;
		var div;

		var switch_value = state.component;

		function switch_props(state) {
			var switch_instance_initial_data = { row: item.data };
			return {
				root: component.root,
				data: switch_instance_initial_data
			};
		}

		if (switch_value) {
			var switch_instance = new switch_value(switch_props(state));
		}

		return {
			key: key_1,

			first: null,

			c: function create() {
				div = createElement("div");
				if (switch_instance) switch_instance._fragment.c();
				this.h();
			},

			h: function hydrate() {
				div.className = "row svelte-1hy6rmt";
				this.first = div;
			},

			m: function mount(target, anchor) {
				insertNode(div, target, anchor);

				if (switch_instance) {
					switch_instance._mount(div, null);
				}
			},

			p: function update(changed, state) {
				item = state.item;
				each_value = state.each_value;
				item_index = state.item_index;
				if (switch_value !== (switch_value = state.component)) {
					if (switch_instance) switch_instance.destroy();

					if (switch_value) {
						switch_instance = new switch_value(switch_props(state));
						switch_instance._fragment.c();
						switch_instance._mount(div, null);
					}
				}

				else {
					var switch_instance_changes = {};
					if (changed.visible) switch_instance_changes.row = item.data;
					switch_instance._set(switch_instance_changes);
				}
			},

			u: function unmount() {
				detachNode(div);
			},

			d: function destroy$$1() {
				if (switch_instance) switch_instance.destroy(false);
			}
		};
	}

	function Index(options) {
		init(this, options);
		this.refs = {};
		this._state = assign(data(), options.data);
		this._recompute({ items: 1, start: 1, end: 1 }, this._state);

		if (!document.getElementById("svelte-1hy6rmt-style")) add_css();

		var _oncreate = oncreate.bind(this);

		if (!options.root) {
			this._oncreate = [];
			this._beforecreate = [];
			this._aftercreate = [];
		}

		this._fragment = create_main_fragment(this, this._state);

		this.root._oncreate.push(_oncreate);

		if (options.target) {
			this._fragment.c();
			this._mount(options.target, options.anchor);

			this._lock = true;
			callAll(this._beforecreate);
			callAll(this._oncreate);
			callAll(this._aftercreate);
			this._lock = false;
		}
	}

	assign(assign(Index.prototype, methods), proto);

	Index.prototype._recompute = function _recompute(changed, state) {
		if (changed.items || changed.start || changed.end) {
			if (this._differs(state.visible, (state.visible = visible(state.items, state.start, state.end)))) changed.visible = true;
		}
	};

	return Index;

})));
